/**
  PasswordMaker - Creates and manages passwords
  Copyright (C) 2005 Eric H. Jung and LeahScape, Inc.
  http://passwordmaker.org/
  grimholtz@yahoo.com

  This library is free software; you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by
  the Free Software Foundation; either version 2.1 of the License, or (at
  your option) any later version.

  This library is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESSFOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
  for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this library; if not, write to the Free Software Foundation,
  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
**/

// References to the UI elements
var clearClipboardCB, clearClipboard, maskGeneratedPasswordCB, pwShortcutLB, hideMPWFieldCB,
  confirmMPWCB, viewPwdFieldsCB, removeAutoCompleteCB, statusbarIndicatorCB;

function initGlobalSettingsPanel() {

  // Select the last-selected tab
  var tabs = document.getElementById("tabs");
  if (tabs) {
    // tb is null if this is the basic options dialog
    tabs.selectedIndex = PwdMkr_RDFUtils.getTarget(passwordMaker.globalSettingsSubjectRes, "selectedTabIndex", 0);  
  }
  
  // Get references to the UI elements
  clearClipboardCB = document.getElementById("clearClipboardCB");
  clearClipboard = document.getElementById("clearClipboard");
  maskGeneratedPasswordCB = document.getElementById("maskGeneratedPasswordCB");
  pwShortcutLB = document.getElementById("pwShortcutLB");
  hideMPWFieldCB = document.getElementById("hideMPWFieldCB");
  viewPwdFieldsCB = document.getElementById("viewPwdFieldsCB");
  removeAutoCompleteCB = document.getElementById("removeAutoCompleteCB");
  statusbarIndicatorCB = document.getElementById("statusbarIndicatorCB");
  
  // Masking
  maskGeneratedPasswordCB.checked = passwordMaker._getLiteral(passwordMaker.globalSettingsSubjectRes, passwordMaker.maskMasterPasswordPredicateRes) == "true";
  
  // Auto-clear clipboard
  clearClipboardCB.checked = passwordMaker._getLiteral(passwordMaker.globalSettingsSubjectRes, passwordMaker.autoClearClipboardPredicateRes) == "true";

  // Auto-clear clipboard secs
  clearClipboard.value = passwordMaker._getLiteral(passwordMaker.globalSettingsSubjectRes, passwordMaker.autoClearClipboardSecondsPredicateRes);

  // Password Shortcut (ALT-`)
  pwShortcutLB.value = passwordMaker._getLiteral(passwordMaker.globalSettingsSubjectRes, passwordMaker.passwordShortcutPredicateRes);
  hideMPWFieldCB.checked = passwordMaker._getLiteral(passwordMaker.globalSettingsSubjectRes, passwordMaker.hideMasterPasswordFieldPredicateRes) == "true";
  if (passwordMaster.value == "") {
    // Can't ever be checked if the mpw is empty
    hideMPWFieldCB.checked = false;
    PwdMkr_RDFUtils.updateTarget(passwordMaker.globalSettingsSubjectRes, "hideMasterPasswordField", "false");
  }
  if (hideMPWFieldCB.checked)
    hideMPWFieldCB.doCommand();
  
  confirmMPWCB.checked = passwordMaker._getLiteral(passwordMaker.globalSettingsSubjectRes, passwordMaker.confirmMPWPredicateRes) == "true";
  viewPwdFieldsCB.checked = passwordMaker.viewPwdFields;
  removeAutoCompleteCB.checked = passwordMaker.removeAutoComplete;
  statusbarIndicatorCB.checked = passwordMaker._getLiteral(passwordMaker.globalSettingsSubjectRes, passwordMaker.statusbarIndicatorPredicateRes) == "true";

  onClearClipboardCB();
}

// Save last-focused tab
function updateTabIndex() {
  // When the dialog if first opened, Mozilla calls this
  // callback fcn but passwdmakerOverlay.init() hasn't been called yet.
  if (passwordMaker.globalSettingsSubjectRes)
    PwdMkr_RDFUtils.updateTarget(passwordMaker.globalSettingsSubjectRes, "selectedTabIndex", document.getElementById("tabs").selectedIndex);
}

function onConfirmMPW() {
  PwdMkr_RDFUtils.updateTarget(passwordMaker.globalSettingsSubjectRes, "confirmMPW", confirmMPWCB.checked ? "true":"false");
  document.getElementById("mpwConfirmationRow").hidden = !confirmMPWCB.checked;
}

function onViewPwdFields() {
  PwdMkr_RDFUtils.updateTarget(passwordMaker.globalSettingsSubjectRes, "viewPwdFields", viewPwdFieldsCB.checked ? "true":"false");
  passwordMaker.viewPwdFields = viewPwdFieldsCB.checked; // update global cached setting
  togglePasswordField();
}

function onRemoveAutoComplete() {
  PwdMkr_RDFUtils.updateTarget(passwordMaker.globalSettingsSubjectRes, "removeAutoComplete", removeAutoCompleteCB.checked ? "true":"false");
  passwordMaker.removeAutoComplete = removeAutoCompleteCB.checked; // update global cached setting  
  var inputElements = _getInputFields(); 
  for (var i=0; i<inputElements.length; i++) {
    if (inputElements[i].type != "password")
      inputElements[i].setAttribute("autocomplete", removeAutoCompleteCB.checked ? "on" : "off");
  }
}

function _getInputFields() {
  // Get recent wnd
  var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService(Components.interfaces.nsIWindowMediator);
  var recentWindow = wm.getMostRecentWindow("navigator:browser");
  if (recentWindow)
    return recentWindow.content.document.getElementsByTagName("input");
  else
    return new Array();
}

/**
 * Similar to passwordMaker.modifyPasswordFieldAttributes(), but that
 * version is optimized for speed since it runs every pageLoad().
 */
function togglePasswordField() {
  var inputElements = _getInputFields(); 
  if (viewPwdFieldsCB.checked) {      
    for (var i=0; i<inputElements.length; i++) {
      if (inputElements[i].type == "password") { // .type is always lower-case 
        inputElements[i].type = "edit";
        inputElements[i].setAttribute("oldtype", "password");
      }
    }
  }
  else {
    for (var i=0; i<inputElements.length; i++) {
      if (inputElements[i].getAttribute("oldtype") == "password") { // .type is always lower-case 
        inputElements[i].type = "password";
        inputElements[i].removeAttribute("oldtype");
      }
    }
  }
}

function onHideMPWField() {
  if (hideMPWFieldCB.checked) {
    if (passwordMaster.value == "") {
      passwordMaker.prompts.alert(this, "PasswordMaker", "You must first enter a master password");
      hideMPWFieldCB.checked = false;
    }
  }
  else
    showMPWField();
  PwdMkr_RDFUtils.updateTarget(passwordMaker.globalSettingsSubjectRes, "hideMasterPasswordField", hideMPWFieldCB.checked ? "true":"false");
}

function showMPWField() {
  // User wants to turn off masking
  // Firefox unchecked the checbkox, even though the user
  // hasn't entered the master pw yet. Check it here so
  // the UI matches the current state of affairs.
  hideMPWFieldCB.checked = true;
  if (promptMPW(passwordMaster.value))
    hideMPWFieldCB.checked = false;
}

/**
 * Method which fires when the Clear Clipboard checkbox
 * has been clicked.
 */
function onClearClipboardCB() {
  clearClipboard.disabled = !clearClipboardCB.checked;
  if (clearClipboardCB.checked && clearClipboard.value == "")
    clearClipboard.value = "10";
  PwdMkr_RDFUtils.updateTarget(passwordMaker.globalSettingsSubjectRes, "autoClearClipboard", clearClipboardCB.checked ? "true":"false");
}

function onClearClipboard() {
  if (/\D/.test(clearClipboard.value))
    clearClipboard.value="10";
  PwdMkr_RDFUtils.updateTarget(passwordMaker.globalSettingsSubjectRes, "autoClearClipboardSeconds", clearClipboard.value);
}

function onPWShortcutLBChanged() {
  PwdMkr_RDFUtils.updateTarget(passwordMaker.globalSettingsSubjectRes, "passwordShortcut", pwShortcutLB.value);
  passwordMaker.passwordShortcut = pwShortcutLB.value;
}

function onMaskGeneratedPasswordButton() {
  if (!maskGeneratedPasswordCB.checked) {
    // User wants to turn off masking
    if (passwordMaster.value != "") {
      // A master password has been set, so we can't allow
      // them to turn off masking just yet
      // Firefox unchecked the checbkox, even though the user
      // hasn't entered the master pw yet. Check it here so
      // the UI matches the current state of affairs.
      maskGeneratedPasswordCB.checked = true;
      // Prompt user for the master password before turning off
      // generated password masking
      if (promptMPW(passwordMaster.value))
        maskGeneratedPasswordCB.checked = false;
    }
  }
  PwdMkr_RDFUtils.updateTarget(passwordMaker.globalSettingsSubjectRes, "maskMasterPassword", maskGeneratedPasswordCB.checked ? "true":"false");
  passwordGenerated.setAttribute("type", maskGeneratedPasswordCB.checked ? "password" : "");
}

function onStatusbarIndicatorCB() {
  pwdMakerStatusBar.toggleShowHide(statusbarIndicatorCB.checked);
}

function onSetSettingsPath() {
  var fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
  fp.init(window, "Select directory and filename for settings", nsIFilePicker.modeSave);
  if (fp.show() == nsIFilePicker.returnOK)
    alert(fp.file.path);
}

function promptMPW(mpw) {
  var okclicked = true;
  try {
    while (okclicked) {
      var input = {value:""};
      okclicked = passwordMaker.prompts.promptPassword(this, "PasswordMaker",
        "For security purposes, you must enter the master password:", input, null, {});
      if (okclicked) {
        if (input.value == mpw)
          return true;
        else
          this.prompts.alert(this, "PasswordMaker", "That is not the correct master password.");
      }
    }
    return false;
  }
  finally {
    mpw = null; // proactively clear sensitive memory
  }
}