/**
  PasswordMaker - Creates and manages passwords
  Copyright (C) 2005 Eric H. Jung and LeahScape, Inc.
  http://passwordmaker.org/
  grimholtz@yahoo.com

  This library is free software; you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by
  the Free Software Foundation; either version 2.1 of the License, or (at
  your option) any later version.

  This library is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESSFOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
  for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this library; if not, write to the Free Software Foundation,
  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
**/

function Field(subjectRes, urlToMatch) {
  this.subjectRes = subjectRes;
  this.urlToMatch = urlToMatch;
}

var passwordMaker = {
  appcontent : null,
  domains : null,
  
  autoPopURLs : null,
  autoPopDefault : false,
  allURLs : null,

  advancedOptionsDialog : "false", // which dialog to open -- advanced options or basic
  passwordShortcut : "pop-all", // off, pop-all, empty-only, clear-all
  whereLeetLBPredicateRes : null,
  leetLevelLBPredicateRes : null,
  hashAlgorithmLBPredicateRes : null,
  passwordLengthPredicateRes : null,
  urlToUsePredicateRes : null, //http://passwordmaker.mozdev.org/rdf#urlToUse
  urlToMatchPredicateRes : null, //http://passwordmaker.mozdev.org/rdf#urlToMatch
  usernameTBPredicateRes : null,
  counterPredicateRes : null,
  namePredicateRes : null,
  descriptionPredicateRes : null,
  protocolCBPredicateRes : null,
  subdomainCBPredicateRes : null,
  domainCBPredicateRes : null,
  pathCBPredicateRes : null, 
  defaultSubjectRes : null, //http://passwordmaker.mozdev.org/defaults
  
  //global settings
  globalSettingsSubjectRes : null, //http://passwordmaker.mozdev.org/globalSettings
  masterPasswordPredicateRes : null, //http://passwordmaker.mozdev.org/rdf#masterPassword
  masterPasswordKeyPredicateRes : null,
  masterPasswordStoragePredicateRes : null,
  maskMasterPasswordPredicateRes : null,
  autoClearClipboardPredicateRes : null,
  autoClearClipboardSecondsPredicateRes : null,
  autoPopulatePredicateRes : null,
  viewPwdFieldsPredicateRes : null,
  removeAutoCompletePredicateRes : null,
  confirmMPWPredicateRes : null,
  statusbarIndicatorPredicateRes : null,
  advancedOptionsDialogPredicateRes : null,
  
  charsetPredicateRes : null,
  prefixPredicateRes : null,
  suffixPredicateRes : null,
  passwordShortcutPredicateRes : null,
  //settingsFilePathPredicateRes : null,
  hideMasterPasswordFieldPredicateRes : null,
  formNamePredRes : null,
  fieldNamePredRes : null,
  fieldValuePredRes : null,
  domainsDatasource : null,
  
  // Cached (along with some other RDF facts) for performance
  viewPwdFields : null,
  removeAutoComplete : null,
  
  statusBarPanel : null,
  statusBarTip : null,
  
  // These are not the "Default Settings" (which a user can change) but rather
  // internal hard-coded defaults.
  DEFAULT_LEET : "off",
  DEFAULT_LEET_LEVEL: 1,
  DEFAULT_HASHALG : "md5",
  DEFAULT_LENGTH : "8",
  DEFAULT_NAME : "Defaults",
  DEFAULT_DESCRIPTION : "Default settings for URLs not elsewhere in this list",
  DEFAULT_PROTOCOL : "false",
  DEFAULT_SUBDOMAIN : "false",
  DEFAULT_DOMAIN : "true",
  DEFAULT_PATH : "false",
  DEFAULT_URLTOUSE : "",
  DEFAULT_URLTOMATCH : "",
  DEFAULT_CHARSET : "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789`~!@#$%^&*()_-+={}|[]\\:\";'<>?,./", // base95 or 93 or whatever
  DEFAULT_PREFIX : "",
  DEFAULT_SUFFIX : "",
  DEFAULT_USERNAME : "",
  DEFAULT_COUNTER : "",
  DEFAULT_AUTOPOPULATE : "false",
  
  remotesContainer : null,
  accountsContainer : null,
  domainsContainer : null,  
  prompts : Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService),  
  mpw : null,
  
  init : function() {  
    this.installIcons();
    this.appcontent = document.getElementById("appcontent");   // Browser    
    this.whereLeetLBPredicateRes = PwdMkr_RDFUtils.makePredicate("whereLeetLB");
    this.leetLevelLBPredicateRes = PwdMkr_RDFUtils.makePredicate("leetLevelLB");
    this.hashAlgorithmLBPredicateRes = PwdMkr_RDFUtils.makePredicate("hashAlgorithmLB");
    this.passwordLengthPredicateRes = PwdMkr_RDFUtils.makePredicate("passwordLength");
    this.urlToUsePredicateRes = PwdMkr_RDFUtils.makePredicate("urlToUse");
    this.urlToMatchPredicateRes = PwdMkr_RDFUtils.makePredicate("urlToMatch");
    this.usernameTBPredicateRes = PwdMkr_RDFUtils.makePredicate("usernameTB");
    this.counterPredicateRes = PwdMkr_RDFUtils.makePredicate("counter");
    this.namePredicateRes = PwdMkr_RDFUtils.makePredicate("name");
    this.descriptionPredicateRes = PwdMkr_RDFUtils.makePredicate("description");
    this.protocolCBPredicateRes = PwdMkr_RDFUtils.makePredicate("protocolCB");
    this.subdomainCBPredicateRes = PwdMkr_RDFUtils.makePredicate("subdomainCB");
    this.domainCBPredicateRes = PwdMkr_RDFUtils.makePredicate("domainCB");
    this.pathCBPredicateRes = PwdMkr_RDFUtils.makePredicate("pathCB");
    this.charsetPredicateRes = PwdMkr_RDFUtils.makePredicate("charset");
    this.prefixPredicateRes = PwdMkr_RDFUtils.makePredicate("prefix");
    this.suffixPredicateRes = PwdMkr_RDFUtils.makePredicate("suffix");
    this.autoPopulatePredicateRes = PwdMkr_RDFUtils.makePredicate("autoPopulate");
  
    this.formNamePredRes = PwdMkr_RDFUtils.makePredicate("formname");
    this.fieldNamePredRes = PwdMkr_RDFUtils.makePredicate("fieldname");
    this.fieldValuePredRes = PwdMkr_RDFUtils.makePredicate("fieldvalue");

    this.defaultSubjectRes = PwdMkr_RDFUtils.makeResource("http://passwordmaker.mozdev.org/defaults");
    
    //global settings
    this.globalSettingsSubjectRes = PwdMkr_RDFUtils.makeResource("http://passwordmaker.mozdev.org/globalSettings");
    this.masterPasswordPredicateRes = PwdMkr_RDFUtils.makePredicate("masterPassword");
    this.masterPasswordKeyPredicateRes = PwdMkr_RDFUtils.makePredicate("masterPasswordKey");
    this.masterPasswordStoragePredicateRes = PwdMkr_RDFUtils.makePredicate("masterPasswordStorage");
    this.maskMasterPasswordPredicateRes = PwdMkr_RDFUtils.makePredicate("maskMasterPassword");
    this.autoClearClipboardPredicateRes = PwdMkr_RDFUtils.makePredicate("autoClearClipboard");
    this.autoClearClipboardSecondsPredicateRes = PwdMkr_RDFUtils.makePredicate("autoClearClipboardSeconds");
    this.advancedOptionsDialogPredicateRes = PwdMkr_RDFUtils.makePredicate("advancedOptionsDialog");
    this.passwordShortcutPredicateRes = PwdMkr_RDFUtils.makePredicate("passwordShortcut");
    //this.settingsFilePathPredicateRes = PwdMkr_RDFUtils.makePredicate("settingsFilePath");
    this.hideMasterPasswordFieldPredicateRes = PwdMkr_RDFUtils.makePredicate("hideMasterPasswordField");
    this.viewPwdFieldsPredicateRes = PwdMkr_RDFUtils.makePredicate("viewPwdFields");
    this.removeAutoCompletePredicateRes = PwdMkr_RDFUtils.makePredicate("removeAutoComplete");    
    this.confirmMPWPredicateRes = PwdMkr_RDFUtils.makePredicate("confirmMPW");
    this.statusbarIndicatorPredicateRes = PwdMkr_RDFUtils.makePredicate("statusbarIndicator");   
    
    this.initGlobals();

    // Initialize containers 
    this.remotesContainer = PwdMkr_RDFUtils.makeSequence("remotes");
    this.accountsContainer = PwdMkr_RDFUtils.makeSequence("accounts");
    this.domainsContainer = PwdMkr_RDFUtils.makeSequence("domains");
    
    // Get MPW if it's been saved to disk
    this.setMPW(new MasterPassword(PwdMkr_RDFUtils.getTarget(this.globalSettingsSubjectRes, this.masterPasswordPredicateRes), // Might be null if user doesn't like to save the master pw
      PwdMkr_RDFUtils.getTarget(this.globalSettingsSubjectRes, this.masterPasswordKeyPredicateRes)));

    // Which dialog is the preferred one (basic or advanced)?
    this.advancedOptionsDialog = PwdMkr_RDFUtils.getTarget(this.globalSettingsSubjectRes, this.advancedOptionsDialogPredicateRes);

    // Password Shortcut (ALT-`)
    this.passwordShortcut = PwdMkr_RDFUtils.getTarget(passwordMaker.globalSettingsSubjectRes, passwordMaker.passwordShortcutPredicateRes);

    this.loadDomains();
    this.loadURLs();

    this.viewPwdFields = PwdMkr_RDFUtils.getTarget(this.globalSettingsSubjectRes, this.viewPwdFieldsPredicateRes) == "true";
    this.removeAutoComplete = PwdMkr_RDFUtils.getTarget(this.globalSettingsSubjectRes, this.removeAutoCompletePredicateRes) == "true";    
    
    // Start listening for pageLoads
    this.appcontent.addEventListener("load", this.pageLoad, true);
    // stop listening
    //this.appcontent.removeEventListener("load", this.pageLoad, true);

    // listen for context-menu
    document.getElementById("contentAreaContextMenu").addEventListener("popupshowing", this.contextPopupShowing, false);
    pwdMakerStatusBar.init();
  },

  /**
   * Set the _masterPassword variable from the supplied argument.
   * This supplied argument must be a MasterPassword object.
   */
  setMPW : function(oMPW) {
    passwordMaker.mpw = oMPW && oMPW.encryptedData && oMPW.key ? oMPW : null;
  },
    
  initGlobals : function() {
    // Get the accounts sequence, or create it if it doesn't exist
    var accountsSubjectRes = PwdMkr_RDFUtils.makeResource("http://passwordmaker.mozdev.org/accounts");
    
    var accountsContainer = PwdMkr_RDFUtils.rdfContainerUtils.MakeSeq(PwdMkr_RDFUtils.datasource, accountsSubjectRes);
    if (PwdMkr_RDFUtils.rdfContainerUtils.IsEmpty(PwdMkr_RDFUtils.datasource, accountsSubjectRes)) {
      // TODO: change the above line so it specifically checks for defaultSubjectRes, not if the container is empty
      // Make the defaults resources.
      PwdMkr_RDFUtils.updateTarget(this.defaultSubjectRes, this.whereLeetLBPredicateRes, this.DEFAULT_LEET);
      PwdMkr_RDFUtils.updateTarget(this.defaultSubjectRes, this.leetLevelLBPredicateRes, this.DEFAULT_LEET_LEVEL);
      PwdMkr_RDFUtils.updateTarget(this.defaultSubjectRes, this.hashAlgorithmLBPredicateRes, this.DEFAULT_HASHALG);    
      PwdMkr_RDFUtils.updateTarget(this.defaultSubjectRes, this.passwordLengthPredicateRes, this.DEFAULT_LENGTH);    
      PwdMkr_RDFUtils.updateTarget(this.defaultSubjectRes, this.namePredicateRes, this.DEFAULT_NAME);    
      PwdMkr_RDFUtils.updateTarget(this.defaultSubjectRes, this.descriptionPredicateRes, this.DEFAULT_DESCRIPTION);    
      PwdMkr_RDFUtils.updateTarget(this.defaultSubjectRes, this.protocolCBPredicateRes, this.DEFAULT_PROTOCOL);    
      PwdMkr_RDFUtils.updateTarget(this.defaultSubjectRes, this.subdomainCBPredicateRes, this.DEFAULT_SUBDOMAIN);    
      PwdMkr_RDFUtils.updateTarget(this.defaultSubjectRes, this.domainCBPredicateRes, this.DEFAULT_DOMAIN);    
      PwdMkr_RDFUtils.updateTarget(this.defaultSubjectRes, this.pathCBPredicateRes, this.DEFAULT_PATH);    
      PwdMkr_RDFUtils.updateTarget(this.defaultSubjectRes, this.charsetPredicateRes, this.DEFAULT_CHARSET);    
      PwdMkr_RDFUtils.updateTarget(this.defaultSubjectRes, this.prefixPredicateRes, this.DEFAULT_PREFIX);    
      PwdMkr_RDFUtils.updateTarget(this.defaultSubjectRes, this.suffixPredicateRes, this.DEFAULT_SUFFIX);    
      PwdMkr_RDFUtils.updateTarget(this.defaultSubjectRes, this.autoPopulatePredicateRes, this.DEFAULT_AUTOPOPULATE);    
      accountsContainer.AppendElement(this.defaultSubjectRes);
    }
  
    // Do the globalsettings exist? Make them if not. @TODO: change this code to use saveGlobals() instead
    if (!PwdMkr_RDFUtils.hasTarget(this.globalSettingsSubjectRes, this.masterPasswordStoragePredicateRes))
      PwdMkr_RDFUtils.updateTarget(this.globalSettingsSubjectRes, this.masterPasswordStoragePredicateRes, "do-not-store");
    if (!PwdMkr_RDFUtils.hasTarget(this.globalSettingsSubjectRes, this.maskMasterPasswordPredicateRes))
      PwdMkr_RDFUtils.updateTarget(this.globalSettingsSubjectRes, this.maskMasterPasswordPredicateRes, "false");
    if (!PwdMkr_RDFUtils.hasTarget(this.globalSettingsSubjectRes, this.autoClearClipboardPredicateRes))
      PwdMkr_RDFUtils.updateTarget(this.globalSettingsSubjectRes, this.autoClearClipboardPredicateRes, "true");
    if (!PwdMkr_RDFUtils.hasTarget(this.globalSettingsSubjectRes, this.autoClearClipboardSecondsPredicateRes))
      PwdMkr_RDFUtils.updateTarget(this.globalSettingsSubjectRes, this.autoClearClipboardSecondsPredicateRes, 10);
    if (!PwdMkr_RDFUtils.hasTarget(this.globalSettingsSubjectRes, this.advancedOptionsDialogPredicateRes))
      PwdMkr_RDFUtils.updateTarget(this.globalSettingsSubjectRes, this.advancedOptionsDialogPredicateRes, "false");
    if (!PwdMkr_RDFUtils.hasTarget(this.globalSettingsSubjectRes, this.passwordShortcutPredicateRes))
      PwdMkr_RDFUtils.updateTarget(this.globalSettingsSubjectRes, this.passwordShortcutPredicateRes, "pop-all"); // 0.6.1    
    if (!PwdMkr_RDFUtils.hasTarget(this.globalSettingsSubjectRes, this.hideMasterPasswordFieldPredicateRes))
      PwdMkr_RDFUtils.updateTarget(this.globalSettingsSubjectRes, this.hideMasterPasswordFieldPredicateRes, "false"); // 0.7.1
    if (!PwdMkr_RDFUtils.hasTarget(this.globalSettingsSubjectRes, this.viewPwdFieldsPredicateRes))
      PwdMkr_RDFUtils.updateTarget(this.globalSettingsSubjectRes, this.viewPwdFieldsPredicateRes, "false"); // 0.8.4
    if (!PwdMkr_RDFUtils.hasTarget(this.globalSettingsSubjectRes, this.removeAutoCompletePredicateRes))
      PwdMkr_RDFUtils.updateTarget(this.globalSettingsSubjectRes, this.removeAutoCompletePredicateRes, "false"); // 0.8.4
    if (!PwdMkr_RDFUtils.hasTarget(this.globalSettingsSubjectRes, this.confirmMPWPredicateRes))
      PwdMkr_RDFUtils.updateTarget(this.globalSettingsSubjectRes, this.confirmMPWPredicateRes, "true"); // 0.8.4
    if (!PwdMkr_RDFUtils.hasTarget(this.globalSettingsSubjectRes, this.statusbarIndicatorPredicateRes))
      PwdMkr_RDFUtils.updateTarget(this.globalSettingsSubjectRes, this.statusbarIndicatorPredicateRes, "true"); // 0.9
    PwdMkr_RDFUtils.flush();
  },
  
  /**
   * Load the list of top-level domains which require second-level
   * domains (e.g., uk) or third, fourth, fifth-level (etc...) domains
   * into this.domains associative array.
   */
  loadDomains : function() {
    var domainIdPredicateRes = PwdMkr_RDFUtils.makePredicate("domainid");
    this.domains = new Array();
    var compositeDS = Components.classes["@mozilla.org/rdf/datasource;1?name=composite-datasource"].getService(Components.interfaces.nsIRDFCompositeDataSource);
    compositeDS.AddDataSource(PwdMkr_RDFUtils.datasource);
    compositeDS.AddDataSource(PwdMkr_RDFUtils.rdfService.GetDataSourceBlocking("chrome://passwdmaker/content/domains.rdf"));
  
    var domains = passwordMaker.domainsContainer.GetElements();
    while (domains.hasMoreElements()) {
      var country = domains.getNext();
      if (country instanceof Components.interfaces.nsIRDFResource) {
        var countryContainer = PwdMkr_RDFUtils.rdfContainerUtils.MakeSeq(compositeDS, country);
        var subdomains = countryContainer.GetElements();
        while (subdomains.hasMoreElements()) {
          var subdomain = subdomains.getNext();
          var t = subdomain;
          if (subdomain instanceof Components.interfaces.nsIRDFResource) {
            if (!PwdMkr_RDFUtils.rdfContainerUtils.IsContainer(compositeDS, subdomain)) {
              // Get the domainid target
              var target = this._getLiteral(t, domainIdPredicateRes); // for passwordmaker.rdf (custom domains)
              if (target == null)
                target = subdomain.Value.substring(subdomain.Value.lastIndexOf("/")+1); // for domains.rdf (default domains)
              this.domains[target] = true;     
            }
          }
        }
      }
    }
  },

  loadURLs : function() {
    // Load all URLs (used during alt-` and context-menu select)  
    passwordMaker.allURLs = new Array();
    var resEnum = PwdMkr_RDFUtils.datasource.GetAllResources();
    while (resEnum.hasMoreElements()) {
      var subjectRes = resEnum.getNext(); // e.g., http://passwordmaker.mozdev.org/yahoo/tom_cruise
      var urlToMatch = PwdMkr_RDFUtils.getTarget(subjectRes, passwordMaker.urlToMatchPredicateRes);
      if (urlToMatch != null && urlToMatch != "")
        passwordMaker.allURLs[passwordMaker.allURLs.length] = new Field(subjectRes, urlToMatch); 
    }    
    passwordMaker._loadAutoPopulateURLs();
  },
  
  _loadAutoPopulateURLs : function() {
    // Load auto-pop URLs set to true (used only during pageloads)  
    passwordMaker.autoPopURLs = new Array();
    passwordMaker.autoPopDefault = false;
    var subjectsRes = PwdMkr_RDFUtils.datasource.GetSources(passwordMaker.autoPopulatePredicateRes,
      PwdMkr_RDFUtils.rdfService.GetLiteral("true"), true);
      
    // Now we have all subjects where autoPopulate=true. Get their urlToMatch values.
    while (subjectsRes.hasMoreElements()) {
      var subjectRes = subjectsRes.getNext();
      if (passwordMaker.defaultSubjectRes == subjectRes)
        passwordMaker.autoPopDefault = true;
      else {
        var urlToMatch = passwordMaker._getLiteral(subjectRes, passwordMaker.urlToMatchPredicateRes);
        if (urlToMatch != null && urlToMatch != "")
          passwordMaker.autoPopURLs[passwordMaker.autoPopURLs.length] = new Field(subjectRes, urlToMatch);
      }
    }
  },

  /**
   * Callback function called by Mozilla when context-menu appears
   */
  contextPopupShowing : function() {
    var target = gContextMenu.target;
    // target can be null when opening something like
    // chrome://passwdmaker/chrome/uris.xul directly in the browser.
    if (target) {
      //var type = target.type; // always lower-case
      document.getElementById("passwordmaker-popup").hidden = 
        !(target.tagName == "INPUT" || target.tagName == "TEXTAREA");
      // Save a reference to the object being clicked
      passwordMaker.clickedElement = gContextMenu.target;
    }
  },

  /**
   * Callback function called by Mozilla when the context-menu
   * item has been clicked
   */
  onContextMenuClick : function() {
    if (passwordMaker.clickedElement) {
      // Get matching accounts (if any)
      var uri = passwordMaker._getURI();
      var matchingSubjectResArray = passwordMaker._findMatches(uri, passwordMaker.allURLs);    
      if (matchingSubjectResArray.length == 0) {
        matchingSubjectResArray.push(passwordMaker.defaultSubjectRes);
        pwdMakerStatusBar.update(passwordMaker.defaultSubjectRes, passwordMaker.readAccountSettings(passwordMaker.defaultSubjectRes));
      }

      // Is it a pw, username, or other field?
      if (passwordMaker.clickedElement.tagName == "INPUT" && passwordMaker.clickedElement.type.match(/text/i) && passwordMaker.clickedElement.name.match(/ID|un|name|user|usr|log|email|mail|acct|ssn/i)) {
        // It's a username field        
        passwordMaker._processRequest(matchingSubjectResArray, {usernames: new Array(passwordMaker.clickedElement), passwords: new Array()});        
      }
      else if(passwordMaker.clickedElement.tagName == "INPUT" && (passwordMaker.clickedElement.type.match(/pass/i) || (passwordMaker.clickedElement.type.match(/text/i) && passwordMaker.clickedElement.name.match(/pass|word|pw/i)))) {
        // It's a password field        
        passwordMaker._processRequest(matchingSubjectResArray, {usernames: new Array(), passwords: new Array(passwordMaker.clickedElement)});      
      }
      else {
        // Other fields
      }     
      passwordMaker.clickedElement = null; // Remove reference to the saved element which was saved in contextPopupShowing()
    }
  },

  /**
   * Called for all populate requests!
   */
  _processRequest : function(matchingSubjectResArray, unpwsubmit, populatationAction) {
    if (!populatationAction)
      populatationAction = "pop-all";
      
    // Prompt user to choose account (if necessary)           
    var matchingSubjectRes = matchingSubjectResArray.length > 1 ?
        passwordMaker._promptUserToChooseAccount(matchingSubjectResArray) : matchingSubjectResArray[0];
    
    if (matchingSubjectRes == null)
      return; // user canceled from _promptUserToChooseAccount()
      
    var username = passwordMaker._getLiteral(matchingSubjectRes, passwordMaker.usernameTBPredicateRes);
    pwdMakerStatusBar.update(matchingSubjectRes, passwordMaker.readAccountSettings(matchingSubjectRes));
        
    if (unpwsubmit.usernames.length > 0) {     
      // Handle usernames      
      if (matchingSubjectRes) {
        passwordMaker._populateUsernames(unpwsubmit.usernames, username, populatationAction);
      }
    }    
    if (unpwsubmit.passwords.length > 0) {
      // Handle passwords        
      if (matchingSubjectRes)  {
        passwordMaker._populatePasswords(unpwsubmit.passwords, matchingSubjectRes, populatationAction);           
      }
    }
  },
  
  /**
   * Coolkey
   */
  _processShortcut : function(doc) {  
    // Check to see if the page has username/password fields
    var unpwsubmit = passwordMaker._getUserPassSubmit(passwordMaker._getAllInputFields(doc));
    if (unpwsubmit.passwords.length == 0)
      return;
          
    if (passwordMaker.passwordShortcut == "clear-all") {
      // Clear the usernames
      for (var i=0; i<unpwsubmit.usernames.length; i++)
        unpwsubmit.usernames[i].value = "";
        
      // Clear the passwords
      for (var i=0; i<unpwsubmit.passwords.length; i++)
        unpwsubmit.passwords[i].value = "";
      return;
    }
    else if (passwordMaker.passwordShortcut == "empty-only") {
      // If there's nothing empty, just return because we don't
      // want to prompt the user to choose an account if there's
      // more than one match.
      var foundEmptyField = false;
      for (var i=0; !foundEmptyField && i<unpwsubmit.usernames.length; i++) {
        if (unpwsubmit.usernames[i].value == "")
          foundEmptyField = true;
      }
      for (var i=0; !foundEmptyField && i<unpwsubmit.passwords.length; i++) {
        if (unpwsubmit.passwords[i].value == "")
          foundEmptyField = true;
      }
      // There are no empty fields to populate, so don't bother continuing
      if (!foundEmptyField)
        return;        
    }
    var matchingSubjectResArray = passwordMaker._findMatches(doc.URL, passwordMaker.allURLs);
    if (matchingSubjectResArray.length == 0) {
      matchingSubjectResArray.push(passwordMaker.defaultSubjectRes);
      pwdMakerStatusBar.update(passwordMaker.defaultSubjectRes, passwordMaker.readAccountSettings(passwordMaker.defaultSubjectRes));
    }
    passwordMaker._processRequest(matchingSubjectResArray, unpwsubmit, passwordMaker.passwordShortcut);
  },  

  /**
   * Callback function called by Mozilla when a page loads
   */
  pageLoad : function(evt) {
    pwdMakerStatusBar.clear();
    
    // doc is document that triggered onload event
    var doc = evt.originalTarget;
    var inputElements = passwordMaker._getAllInputFields(doc);
     
    var matchingSubjectResArray = passwordMaker._findMatches(doc.URL, passwordMaker.autoPopURLs);
    if (matchingSubjectResArray.length == 0 && passwordMaker.autoPopDefault) {
      matchingSubjectResArray.push(passwordMaker.defaultSubjectRes);
      pwdMakerStatusBar.update(passwordMaker.defaultSubjectRes, passwordMaker.readAccountSettings(passwordMaker.defaultSubjectRes));
    }      
    if (matchingSubjectResArray.length > 0) {
      // Check to see if the page has username/password fields
      var unpwsubmit = passwordMaker._getUserPassSubmit(inputElements);
      if (unpwsubmit.passwords.length > 0)
        passwordMaker._processRequest(matchingSubjectResArray, unpwsubmit);
    }
    passwordMaker.modifyFieldAttributes(inputElements);  
  },
  
  getCalculatedURIUsedForDefault : function() {
    var defaults = passwordMaker.readAccountSettings(passwordMaker.defaultSubjectRes);
    return passwordMaker.calculateURI(defaults.protocol == "true",
                                     defaults.subdomain == "true",
                                     defaults.domain == "true",
                                     defaults.path == "true", //@TODO: change this to port when supported in the GUI
                                     defaults.path == "true", window['passwordmaker-uri']);
  },

  modifyFieldAttributes : function(inputElements) {
    if (passwordMaker.viewPwdFields || passwordMaker.removeAutoComplete) {     
      for (var i=0; i<inputElements.length; i++) {
        if (inputElements[i].type == "password")  { // .type is always lower-case
          if (passwordMaker.viewPwdFields) {
            inputElements[i].type = "edit";
            inputElements[i].setAttribute("oldtype", "password");
          }
        }
        else if (passwordMaker.removeAutoComplete) {
          // Don't turn on auto-complete for password fields! (security)
          inputElements[i].setAttribute("autocomplete", "on");
        }
      }    
    }
  },
  
  _getAllInputFields : function(doc) {
    var inputs = doc.getElementsByTagName("input");
    var ret = new Array();
    for (var i=0; i<inputs.length; i++)
      ret.push(inputs[i]);
    var f = doc.getElementsByTagName("frame");
    for (var i=0; i<f.length; i++) {
      inputs = f[i].contentDocument.getElementsByTagName("input");        
      for (var j=0; j<inputs.length; j++)
        ret.push(inputs[j]);
    }
    return ret;
  },
  
  /**
   * Called on alt-` keyboard click
   */
  populateFields : function() {
    // off, pop-all, empty-only, clear-all
    if (passwordMaker.passwordShortcut == "off")
      return;

    var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService(Components.interfaces.nsIWindowMediator);
    var recentWindow = wm.getMostRecentWindow("navigator:browser");
    if (recentWindow) {
      passwordMaker._processShortcut(recentWindow.content.document);
    }
  },
    
  /**
   * Given a subject and predicate, return the literal
   * of the associated target (we're assuming there's
   * only a single target). If the target doesn't exist
   * or isn't a literal, return defaultVal (which often
   * may be null).
   */
  _getLiteral : function(subjectRes, predicateRes, defaultVal) {
    var targetRes = PwdMkr_RDFUtils.datasource.GetTarget(subjectRes, predicateRes, true);
    return targetRes instanceof Components.interfaces.nsIRDFLiteral ? targetRes.Value : defaultVal;
  },
  
  readAccountSettings : function(subjectRes) {
    var ret = new Array();
    ret.name = passwordMaker._getLiteral(subjectRes, passwordMaker.namePredicateRes, "");
    ret.description = passwordMaker._getLiteral(subjectRes, passwordMaker.descriptionPredicateRes, "");
    ret.hashAlgorithm = passwordMaker._getLiteral(subjectRes, passwordMaker.hashAlgorithmLBPredicateRes, passwordMaker.DEFAULT_HASHALG);
    ret.whereLeet = passwordMaker._getLiteral(subjectRes, passwordMaker.whereLeetLBPredicateRes, passwordMaker.DEFAULT_LEET);
    ret.leetLevel = passwordMaker._getLiteral(subjectRes, passwordMaker.leetLevelLBPredicateRes, passwordMaker.DEFAULT_LEET_LEVEL);
    ret.passwordLength = passwordMaker._getLiteral(subjectRes, passwordMaker.passwordLengthPredicateRes, passwordMaker.DEFAULT_LENGTH);
    if (subjectRes == passwordMaker.defaultSubjectRes) {
      ret.protocol = passwordMaker._getLiteral(subjectRes, passwordMaker.protocolCBPredicateRes, passwordMaker.DEFAULT_PROTOCOL);
      ret.subdomain = passwordMaker._getLiteral(subjectRes, passwordMaker.subdomainCBPredicateRes, passwordMaker.DEFAULT_SUBDOMAIN);
      ret.domain = passwordMaker._getLiteral(subjectRes, passwordMaker.domainCBPredicateRes, passwordMaker.DEFAULT_DOMAIN);
      ret.path = passwordMaker._getLiteral(subjectRes, passwordMaker.pathCBPredicateRes, passwordMaker.DEFAULT_PATH);
    }
    else {
      ret.urlToMatch = passwordMaker._getLiteral(subjectRes, passwordMaker.urlToMatchPredicateRes, passwordMaker.DEFAULT_URLTOMATCH);
      ret.urlToUse = passwordMaker._getLiteral(subjectRes, passwordMaker.urlToUsePredicateRes, passwordMaker.DEFAULT_URLTOUSE); 
    }
    ret.charset = passwordMaker._getLiteral(subjectRes, passwordMaker.charsetPredicateRes, passwordMaker.DEFAULT_CHARSET);
    ret.prefix = passwordMaker._getLiteral(subjectRes, passwordMaker.prefixPredicateRes, passwordMaker.DEFAULT_PREFIX);
    ret.suffix = passwordMaker._getLiteral(subjectRes, passwordMaker.suffixPredicateRes, passwordMaker.DEFAULT_SUFFIX);
    ret.username = passwordMaker._getLiteral(subjectRes, passwordMaker.usernameTBPredicateRes, passwordMaker.DEFAULT_USERNAME);
    ret.counter = passwordMaker._getLiteral(subjectRes, passwordMaker.counterPredicateRes, passwordMaker.DEFAULT_COUNTER);
    ret.autoPopulate = passwordMaker._getLiteral(subjectRes, passwordMaker.autoPopulatePredicateRes, passwordMaker.DEFAULT_AUTOPOPULATE);
    return ret;  
  },
  
  /**
   * Generate the site-specific password and return it. If there are any
   * problems decrypting the master password, throw an exception.
   */
  _generateSiteSpecificPassword : function(selectedSubjectRes) {

    var decryptedMasterPassword = PwdMkr_MPW.getDecryptedMPW(passwordMaker.mpw);
    if (decryptedMasterPassword == null)
      return null; // user might simply have hit the cancel button when prompted.

    var ret = passwordMaker.readAccountSettings(selectedSubjectRes);

    var uri = "";
    if (selectedSubjectRes == passwordMaker.defaultSubjectRes) {
      // Determine which part(s) of the URI to use in the password generation algorithm
      uri = passwordMaker.calculateURI(ret.protocol == "true",
              ret.subdomain == "true",
              ret.domain == "true",
              ret.path == "true", //@TODO: change this to port when supported in the GUI
              ret.path == "true");                                        
    }
    else
      uri = ret.urlToUse;

    // Generate the site-specific password
    var ret = passwordMaker.generatepassword(ret.hashAlgorithm, decryptedMasterPassword,
                          uri + ret.username + ret.counter, ret.whereLeet, ret.leetLevel,
                          ret.passwordLength, ret.charset, ret.prefix, ret.suffix);

    // Just to be safe, don't wait for the garbage collector to reclaim the clear-text master password in memory.
    // Even though the variable is on the calling stack (not the heap) and probably doesn't get garbage-collected
    // (I just don't know enough about Mozilla's JavaScript interpreter) -- just tossed -- we set the value to null.
    // This also ensures the stack frame contents -- which probably still exist after the function returns (at least
    // until function calls of the same depth occur) -- can't be inspected to find the decrypted password.
    decryptedMasterPassword = null;
    return {password:ret, subjectRes:selectedSubjectRes}; 
  },
    
  _findMatches : function(url, urlsArray) {    
    var matches = new Array();
    for (var i=0; url && i<urlsArray.length; i++) {
      if (url.indexOf(urlsArray[i].urlToMatch) > -1)
        matches.push(urlsArray[i].subjectRes);
    }
    return matches;
  },
  
  _getUserPassSubmit : function(inputElements) {
    var ret = {usernames: new Array(), passwords: new Array()};
    for (var i=0; i<inputElements.length; i++) {
      if (inputElements[i].type == "password" || inputElements[i].getAttribute("oldtype") == "password") // .type is always lower-case
        ret.passwords[ret.passwords.length] = inputElements[i];
      else if (inputElements[i].type.match(/text/i) && inputElements[i].name.match(/ID|un|name|user|usr|log|email|mail|acct|ssn/i))
        ret.usernames[ret.usernames.length] = inputElements[i];
      /*else if (inputElements[i].type.match(/submit|button|image/i) && (inputElements[i].src.match(/log|sub|sign/i) || inputElements[i].value.match(/log|sub|sign/i) || inputElements[i].name.match(/log|sub|sign/i)))
        ret.submitBtns[ret.submitBtns.length] = inputElements[i];
      }*/
    }
    return ret;
  },
  
  _populateUsernames : function(unFields, username, action) {      
    if (username == null)
      return;
    for (var i=0; i<unFields.length; i++) {
      if (action == "pop-all" || (action == "empty-only" && unFields[i].value == ""))
        unFields[i].value = username;
    }
  },

  /**
   * Populate the password fields in _doc_
   * with the generated password.
   */
  _populatePasswords : function(pwFields, subjectRes, action) {
    var matchingFact = null;
    for (var i=0; i<pwFields.length; i++) {
      if (action == "pop-all" || (action == "empty-only" && pwFields[i].value == "")) {
        // generate the password for this page if it hasn't already been generated
        if (matchingFact == null) {
          matchingFact = passwordMaker._generateSiteSpecificPassword(subjectRes);
          if (matchingFact == null) // user clicked cancel
            return;
        }
        pwFields[i].value = matchingFact.password;
      }
    }

    /*if (matchingFact) {
      // Look for other fields to pop 
      matchingFact.subjectRes = matchingFact.subjectRes.QueryInterface(Components.interfaces.nsIRDFResource);
      //dump("subjectRes=" + matchingFact.subjectRes.Value + "\n");
      passwordMaker._populateUsername(matchingFact.subjectRes);
      var subjectRes = PwdMkr_RDFUtils.rdfService.GetResource(matchingFact.subjectRes.Value + "/fields");
      if (PwdMkr_RDFUtils.rdfContainerUtils.IsEmpty(PwdMkr_RDFUtils.datasource, subjectRes))
        return;
      var fieldsContainer = PwdMkr_RDFUtils.rdfContainerUtils.MakeSeq(PwdMkr_RDFUtils.datasource, subjectRes);
      var autoFillFields = {};
      var fieldsContainterIter = fieldsContainer.GetElements();
      while (fieldsContainterIter.hasMoreElements()) {
        var fieldSubj = fieldsContainterIter.getNext();
        if (fieldSubj instanceof Components.interfaces.nsIRDFResource) {
          var name = passwordMaker._getLiteral(fieldSubj, passwordMaker.fieldNamePredRes);
          var value = passwordMaker._getLiteral(fieldSubj, passwordMaker.fieldValuePredRes);
          autoFillFields[name] = value;
          //dump("autoFillFields[" + name + "] = " + value + "\n");
        }
      }
      for (i in autoFillFields) {
        dump("i=" + i + "\n");
        for (var j=0;j<inputElements.length; j++) {
          //dump(inputElements[j].name + " " + inputElements[j].type + "\n");
          if (inputElements[j].name == i) {
            inputElements[j].value = autoFillFields[i];
            dump("found it\n");
            break;
          }
        }
      }
    }*/
  },
    
  _promptUserToChooseAccount : function(subjectRes) {
    var selectedSubjectRes;
    //dump("subjectRes length is " + subjectRes.length + "\n");
    if (subjectRes.length == 1)
      selectedSubjectRes = subjectRes[0];
    else {
      // There are multiple accounts for this URL. Prompt the user for which one to use
      var params = {inn:{accounts : subjectRes, passwordMaker : passwordMaker}, out:null};
      window.openDialog("chrome://passwdmaker/chrome/chooseaccount.xul", "",
          "chrome, dialog, modal, resizable=yes", params).focus();
      if (params.out) {
        // User clicked OK
        selectedSubjectRes = params.out.selection;
      }
      else {
        // User clicked cancel
        return null;
      }
    }
    return selectedSubjectRes;
  },

  /**
   * Open the main PasswordMaker dialog
   */
  open : function() {
    // If there's a passwordmaker dialog window already open,
    // just focus it and return.
    var wnd = this.findWindow();
    if (wnd)
      wnd.focus();
    else {
      // Open passwordmaker dialog.
      // We don't really need a 10 msec delay here, but this lets us
      // easily re-use all the code in the _open() method.
      setTimeout(this._open, 10);
    }
  },
  
  /**
   * Find and return the PasswordMaker dialog
   */
  findWindow : function() {
    var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService(Components.interfaces.nsIWindowMediator);
    var e = wm.getEnumerator("passwordmaker");
    return e.hasMoreElements() ? e.getNext() : null;
  },

  /**
   * setTimeout() callback function.
   */
  _open : function() {
    // Basic options dialog is resizable because we open it with
    // a specific size in pixels (how to specify in ems?) The entire
    // dialog contents use ems, but if we open in pixels it could look
    // weird on some resolutions/font sizes, so we give the user the
    // option to resize it. Opening the dialog in ems would get around
    // this problem and we could make resizable false.
    var xulfilename = passwordMaker.advancedOptionsDialog == "true" ? "chrome://passwdmaker/content/advanced-options.xul" : "chrome://passwdmaker/content/basic-options.xul";
    passwordMaker.dlg = window.openDialog(xulfilename, "", "minimizable,dialog,chrome,resizable=yes");
    passwordMaker.dlg.focus();
  },

  /**
   * Switch from basic options to advanced, or vice-versa.
   * Update the preferences.
   */
  switchOptionsDialog : function() {
    this.advancedOptionsDialog = this.advancedOptionsDialog == "true" ? "false" : "true";
    PwdMkr_RDFUtils.updateTarget(passwordMaker.globalSettingsSubjectRes, "advancedOptionsDialog", this.advancedOptionsDialog);

    // Inspired by delayedOpenWindow() in browser.js.
    setTimeout(this._open, 10);
  },

  /**
   * Called by calculateURI()
   */
  _parseURI : function(aURI) {
    var temp = aURI.match("([^://]+://)([^/:]*):?([0-9]*)(.*)");  // \d isn't working in place of [0-9], why not?
    if (!temp)
      throw this;
    return { protocol: temp[1],
             domains : temp[2].split("."), // temp[2] is the url without protocol, port, path, query parms, anchor
             port    : temp[3],
             path    : temp[4]};
  },

  _getURI : function() {
    var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService(Components.interfaces.nsIWindowMediator);
    var recentWindow = wm.getMostRecentWindow("navigator:browser");
    if (recentWindow)
      return new String(recentWindow.content.document.location);
    else
      return "";
  },

  calculateURI : function(includeProtocol, includeSubdomain, includeDomain, includePort, includePath, aURI) {
    if (!aURI)
      aURI = passwordMaker._getURI();
      
    if (includeProtocol && includeSubdomain && includeDomain && includePath && includePath)
      return aURI;

    if (!includeProtocol && !includeSubdomain && !includeDomain && !includePath && !includePath)
      return "";
    
    var uriComponents;
    try {
      uriComponents = this._parseURI(aURI);
    }
    catch (e) {
      return aURI; // aURI isn't parsable; e.g., about:config, javascript:, about:blank
    }

    var ret = includeProtocol ? uriComponents.protocol : ""; // set the protocol or empty string

    // Check if, for example, domains['co.uk'] == true
    var TLDrequiresSLD = eval("passwordMaker.domains['" + uriComponents.domains[uriComponents.domains.length-2] + "." +
                            uriComponents.domains[uriComponents.domains.length-1] + "']");


    // TODO: logic for third-level domains !

    if (includeSubdomain) {
      // The subdomains are all domains except the final two, unless
      // the TLD is a ccTLD which requires a SLD. In the latter case,
      // the subdomains are all domains except the final three.
      var len = TLDrequiresSLD ? uriComponents.domains.length-3 : uriComponents.domains.length-2;
      for (var i=0; i<len; i++) {
        ret += uriComponents.domains[i];
        // Add a dot if this isn't the final subdomain
        if (i+1 < len)
          ret += ".";
      }			
    }

    if (includeDomain) {
      if (ret != "" && ret[ret.length-1] != "." && ret[ret.length-1] != "/")
        ret += ".";

      if (uriComponents.domains.length == 1) {
        // e.g., localhost (no .com or other domains)
        ret += uriComponents.domains[0];
      }
      else {
        // The domain for a ccTLD which requires SLD is really SLD+ccTLD;
        // e.g., ac.uk instead of just .uk
        if (TLDrequiresSLD)
          ret += uriComponents.domains[uriComponents.domains.length-3] + ".";
        ret += uriComponents.domains[uriComponents.domains.length-2] + "." + uriComponents.domains[uriComponents.domains.length-1];
      }
    }

    if (includePort)
      ret += ":" + uriComponents.port;

    if (includePath)
      ret += uriComponents.path;

    return ret;
  },
  
  onFocus : function() {
    //var wnd = this.findWindow();
    //if (wnd)   
      //wnd.updateURLLabel();
  },
  
  refresh : function() {
    PwdMkr_RDFUtils.datasource.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource);
    PwdMkr_RDFUtils.datasource.Refresh(true);
    passwordMaker.init();
  },

  /**
   * Generate the password from the key, a hash algorithm,
   * and (optionally) applying l33t speak afterwards. If we're
   * using an HMAC algorithm, we need a key and data, not just a key.
   */
  _generatepassword : function(hashAlgorithm, key, data, whereToUseL33t, l33tLevel, passwordLength, charset, prefix, suffix) {
    
    // Never *ever, ever* allow the charset's length<2 else
    // the hash algorithms will run indefinitely
    if (charset == null || charset.length < 2)
      charset = passwordMaker.DEFAULT_CHARSET;
    if (hashAlgorithm.indexOf("0.6") > -1)
      charset = "0123456789abcdef";
  
    /*dump("Generating password for:\n");
    dump("   hashAlgorithm=" + hashAlgorithm + "\n");
    dump("   key=" + key + "\n");
    dump("   data=" + data + "\n");
    dump("   whereToUseL33t=" + whereToUseL33t + "\n");
    dump("   l33tLevel=" + l33tLevel + "\n");
    dump("   passwordLength=" + passwordLength + "\n");
    dump("   charset=" + charset + "\n");
    dump("   prefix=" + prefix + "\n");
    dump("   suffix=" + suffix + "\n");*/
  
    // for non-hmac algorithms, the key is master pw and url concatenated
    var usingHMAC = hashAlgorithm.indexOf("hmac") > -1;
    if (!usingHMAC)
      key += data; 
  
    // apply l33t before the algorithm?
    if (whereToUseL33t == "both" || whereToUseL33t == "before-hashing") {
      key = PasswordMaker_l33t.convert(l33tLevel, key);
      if (usingHMAC)
        data = PasswordMaker_l33t.convert(l33tLevel, data); // new for 0.3; 0.2 didn't apply l33t to _data_ for HMAC algorithms
    }
  
    // apply the algorithm
    var password;
    switch(hashAlgorithm) {
      case "sha256":
        password = PasswordMaker_SHA256.any_sha256(key, charset);
        break;
      case "hmac-sha256":
        password = PasswordMaker_SHA256.any_hmac_sha256(key, data, charset);
        break;
      case "sha1":
        password = PasswordMaker_SHA1.any_sha1(key, charset);
        break;
      case "hmac-sha1":
        password = PasswordMaker_SHA1.any_hmac_sha1(key, data, charset);
        break;
      case "md4":
        password = PasswordMaker_MD4.any_md4(key, charset);
        break;
      case "hmac-md4":
        password = PasswordMaker_MD4.any_hmac_md4(key, data, charset);
        break;
      case "md5":
        password = PasswordMaker_MD5.any_md5(key, charset);
        break;
      case "md5-v0.6":
        password = PasswordMaker_MD5_V6.hex_md5(key);
        break;      
      case "hmac-md5":
        password = PasswordMaker_MD5.any_hmac_md5(key, data, charset);
        break;
      case "hmac-md5-v0.6":
        password = PasswordMaker_MD5_V6.hex_hmac_md5(key, data);
        break;       
      case "rmd160":
        password = PasswordMaker_RIPEMD160.any_rmd160(key, charset);
        break;
      case "hmac-rmd160":
        password = PasswordMaker_RIPEMD160.any_hmac_rmd160(key, data, charset);
        break;
      default:
        password = "coding error; contact author";
        return;
    }
  
    // apply l33t after the algorithm? 
    if (whereToUseL33t == "both" || whereToUseL33t == "after-hashing")
      password = PasswordMaker_l33t.convert(l33tLevel, password);
    return password;
  },

  /**
   * Wrapper around _generatepassword() which calls it
   * n times in order to support passwords of arbitrary length.
   * Introduced in version 1.3.3.
   */
  generatepassword : function(hashAlgorithm, key, data, whereToUseL33t, l33tLevel, passwordLength, charset, prefix, suffix) {
    var password = "";
    var count = 0;
    while (password.length < passwordLength) {
      // To maintain backwards compatibility with all previous versions of passwordmaker,
      // the first call to _generatepassword() must use the plain "key".
      // Subsequent calls add a number to the end of the key so each iteration
      // doesn't generate the same hash value.
      password += (count == 0) ?
        passwordMaker._generatepassword(hashAlgorithm, key, data, whereToUseL33t, l33tLevel, passwordLength, charset, prefix, suffix) :
        passwordMaker._generatepassword(hashAlgorithm, key + '\n' + count, data, whereToUseL33t, l33tLevel, passwordLength, charset, prefix, suffix);
      count++;
    }
    if (prefix)
      password = prefix + password;
    if (suffix) {
      var pw = password.substring(0, passwordLength-suffix.length) + suffix;
      return pw.substring(0, passwordLength);
    }
    //dump("   generated password=" + password.substring(0, passwordLength) + "\n\n");
    return password.substring(0, passwordLength);
  },
  
  /**
   * Launch the native application associated with mailto: links.
   */
  sendMail : function(mailtoUrl) {
    // Copied from browser.js's MailIntegration class
  
    var ioService = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
    var uri = ioService.newURI(mailtoUrl, null, null);
  
    // Now pass this url to the operating system
    // a generic method which can be used to pass arbitrary urls to the operating system.
    // aURL --> a nsIURI which represents the url to launch
    var extProtocolSvc = Components.classes["@mozilla.org/uriloader/external-protocol-service;1"].getService(Components.interfaces.nsIExternalProtocolService);
    if (extProtocolSvc)
        extProtocolSvc.loadUrl(uri);
  },

  /**
   * Open URL in a tab in a browser, but re-use an existing tab for a given URL if it's already
   * open (prevents the proliferation of many tabs for the same URL, even if the URL is chrome).
   * This code is used by both the about box and the passwdmaker dialog box.
   *
   * Not fully working.
   */
  openAndReuseOneTabPerURL : function(url) {
    //TODO: use technique at http://forums.mozillazine.org/viewtopic.php?t=343929
    //instead of this stuff which never even worked in both Firefox AND Mozilla...
    
    // I wrote this up at http://kb.mozillazine.org/Reusing_tabs_for_the_same_URL
    
    /*var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService(Components.interfaces.nsIWindowMediator);
    var browserEnumerator = wm.getEnumerator("navigator:browser");
  
    // Check each browser instance for our URL
    var found = false;
    while (browserEnumerator.hasMoreElements() && !found) {
      var browserInstance = browserEnumerator.getNext().document.getElementById("content"); 
      // Check each tab of this browser instance
      var currentTab, index = 0, numTabs = browserInstance.mPanelContainer.childNodes.length;
      while (index < numTabs && !found) {
        currentTab = browserInstance.getBrowserAtIndex(index);
        if (url == currentTab.currentURI.spec) {
          // The URL is already opened. Select its tab.
          browserInstance.selectedTab = currentTab;
          // Focus *this* browser
          browserInstance.focus();
          found = true;
        }
        index++;
      }
    }
  
    // Our URL isn't open. Open it now.
    if (!found) {
      var recentWindow = wm.getMostRecentWindow("navigator:browser");
      if (recentWindow) {
        // Use an existing browser window
        recentWindow.delayedOpenTab(url);
      }
      else {
        // No browser windows are open, so open a new one.
        window.open(url);
      }
    }*/
    var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService(Components.interfaces.nsIWindowMediator);
    var recentWindow = wm.getMostRecentWindow("navigator:browser");
    if (recentWindow) {
      // Use an existing browser window
      //recentWindow.delayedOpenTab(url);
      recentWindow.gBrowser.selectedTab = recentWindow.gBrowser.addTab(url);
      return recentWindow.gBrowser.selectedTab;    
    }
    else {
      // No browser windows are open, so open a new one.
      return window.open(url);
    }
  },
  
  /**
   * Function for displaying dialog box with yes/no buttons (not OK/Cancel buttons),
   * or any arbitrary button labels.
   */
  ask : function(parent, text, btn1Text, btn2Text) {
    if (!btn1Text)
      btn1Text = "Yes";
    if (!btn2Text)
      btn2Text = "No";
    return passwordMaker.prompts.confirmEx(parent, "PasswordMaker", text,
      passwordMaker.prompts.BUTTON_TITLE_IS_STRING * passwordMaker.prompts.BUTTON_POS_0 +
      passwordMaker.prompts.BUTTON_TITLE_IS_STRING * passwordMaker.prompts.BUTTON_POS_1,
      btn1Text, btn2Text, null, null, {}) == 0; // 0 means first button ("yes") was pressed
  },

  /**
   * Icon moving code (workaround for EM bug 242757).
   * This makes our icon appear on our dialog window
   * (otherwise the default icon is used).
   *
   * Windows And Linux only for now. Not sure what is needed for OSX.
   *
   * Thanks go out to asqueella and JedBrown for the QuickNote extension
   * and this help:
   * http://forums.mozillazine.org/viewtopic.php?t=219952
   */
  installIcons : function() {
    var iconFileName;
    var platform = navigator.platform.toLowerCase();
  
    if (platform.indexOf("win32") != -1)
      iconFileName = "passwordmakerdlg.ico";
    else if (platform.indexOf("linux") != -1)
      iconFileName = "passwordmakerdlg.xpm";
    else
      iconFileName = "passwordmakerdlg.ico"; // don't know what yet to do on Mac and other platforms
  
    // Target directory for icons is %MozDir%/chrome/icons/default/
    var destDir = PWDMKRDirIO.get("AChrom");
    destDir.append("icons");
    destDir.append("default");
  
    var destFile = destDir.clone();
    destFile.append(iconFileName);
  
    // Original file is at %Profile%/extensions/{5872365E-67D1-4AFD-9480-FD293BEBD20D}/defaults/_filename_
    var origDir = PWDMKRDirIO.get("ProfD");
    origDir.append("extensions");
    origDir.append("{5872365E-67D1-4AFD-9480-FD293BEBD20D}");
    origDir.append("defaults");
    var origFile = origDir.clone();
    origFile.append(iconFileName);
    passwordMaker.installFile(destFile, destDir, origFile);
  },
  
  installFile : function(destFile, destDir, origFile) {
    try {
      if (!destFile.exists() && origFile.exists())
        origFile.copyTo(destDir, null);
    }
    catch (e) {
      dump(e);
    }
  }  
};