/**
  PasswordMaker - Creates and manages passwords
  Copyright (C) 2005 Eric H. Jung and LeahScape, Inc.
  http://passwordmaker.org/
  grimholtz@yahoo.com

  This library is free software; you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by
  the Free Software Foundation; either version 2.1 of the License, or (at
  your option) any later version.

  This library is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESSFOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
  for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this library; if not, write to the Free Software Foundation,
  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
**/

var printWindow;
var isExporting; // true if exporting to HTML/RDF; false if printing
function printSettings() {
  var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService(Components.interfaces.nsIWindowMediator);
  var win = wm.getMostRecentWindow("navigator:browser");
  printWindow = wm.getMostRecentWindow("navigator:browser");
  printWindow.gBrowser.selectedTab = printWindow.gBrowser.addTab("about:blank");
  printWindow.gBrowser.selectedBrowser.addEventListener("load",
    function() {
      var doc = printWindow.gBrowser.contentDocument;
      doc.removeChild(doc.getElementsByTagName("html")[0]);
      isExporting = false;
      var dom = new DOMParser().parseFromString(rdfToHTML(), "text/xml");    
      doc.appendChild(dom.documentElement);
      setTimeout(function() {
        // Display the print dialog -- thanks http://lxr.mozilla.org/aviary101branch/source/toolkit/components/printing/content/printUtils.js
        var printSettings, webBrowserPrint;
        try {      
          webBrowserPrint = printWindow._content.QueryInterface(Components.interfaces.nsIInterfaceRequestor)
            .getInterface(Components.interfaces.nsIWebBrowserPrint);
          // Get print settings
          var PSSVC = Components.classes["@mozilla.org/gfx/printsettings-service;1"]
                              .getService(Components.interfaces.nsIPrintSettingsService);
          printSettings = PSSVC.globalPrintSettings;
          if (!printSettings.printerName)
            printSettings.printerName = PSSVC.defaultPrinterName;
      
          // First get any defaults from the printer 
          PSSVC.initPrintSettingsFromPrinter(printSettings.printerName, printSettings);
          // now augment them with any values from last time
          PSSVC.initPrintSettingsFromPrefs(printSettings, true, printSettings.kInitSaveAll);        
        }
        catch (e) {
          dump(e + "\n");
          return;
        }
        try {
          webBrowserPrint.print(printSettings, null);
        } catch (e) {
          // Pressing cancel is expressed as an NS_ERROR_ABORT return value,
          // causing an exception to be thrown which we catch here.
          // Unfortunately this will also consume helpful failures, so add a
          // dump("print: "+e+"\n"); // if you need to debug
        }}, 2000);
    }, true);  
}

function rdfToHTML() {
  var includeMPW = false;
  if (passwordMaker.mpw)
    includeMPW = passwordMaker.ask(this, "Would your like to include the master password?");
  var incGenPwds = passwordMaker.ask(this, "Would your like to include generated passwords for each account?");
  var incRemPwds = passwordMaker.ask(this, "Would your like to include passwords for remote settings?");  

  var content =
    "<html xmlns=\"http://www.w3.org/1999/xhtml\"><head><title>PasswordMaker. One Password To Rule Them All.</title><link rel=\"icon\" href=\"http://passwordmaker.org/favicon.ico\"/><link rel=\"shortcut icon\" href=\"http://passwordmaker.org/favicon.ico\"/><link rel=\"stylesheet\" href=\"";
    content += isExporting ? "http://passwordmaker.org/styles/print.css\"" : "chrome://passwdmaker/content/print.css\"";
    
  content += " type=\"text/css\"/></head><body><div id=\"canvasWrapper\"><div id=\"canvas\"><a href=\"http://passwordmaker.org/\"><img src=\"http://passwordmaker.org/images/pageheader.jpg\" border=\"0\"/></a><div id=\"contentWrapper\"><div id=\"content\"><div id=\"insertAfter\" class=\"standard\"/>";
  
  if (includeMPW)
    content += "<div class=\"title\">Master Password</div><span class=\"mpwVal\">" +
      PwdMkr_MPW.getDecryptedMPW(passwordMaker.mpw) + "</span>";

  content += _printContainer("Account Settings", "accounts", incGenPwds, _buildAccountAsHTML);
  content += _buildGlobalSettings();
  content += _printContainer("Remote Settings", "remotes", incRemPwds, _buildRemoteAsHTML);
  return content + "</div></div></div></div></body></html>";
}

function _buildGlobalSettings() {
  var content = "<br/><div class=\"title\">Global Settings</div>";
  content += _buildRDFLiteralAsHTML(passwordMaker.globalSettingsSubjectRes, 
    passwordMaker.maskMasterPasswordPredicateRes, "Mask Generated Password with Asterisks", "globalSetting", "globalSettingVal");
  content += _buildRDFLiteralAsHTML(passwordMaker.globalSettingsSubjectRes, 
    passwordMaker.hideMasterPasswordFieldPredicateRes, "Hide Master Password Field (number of asterisks)", "globalSetting", "globalSettingVal");    
  content += _buildRDFLiteralAsHTML(passwordMaker.globalSettingsSubjectRes, 
    passwordMaker.confirmMPWPredicateRes, "Confirm master password by typing it twice instead of once", "globalSetting", "globalSettingVal");      
  content += _buildRDFLiteralAsHTML(passwordMaker.globalSettingsSubjectRes, 
    passwordMaker.viewPwdFieldsPredicateRes, "Show all passwords on web pages as clear text", "globalSetting", "globalSettingVal");     
  content += _buildRDFLiteralAsHTML(passwordMaker.globalSettingsSubjectRes, 
    passwordMaker.removeAutoCompletePredicateRes, "Enable browser auto-complete on webpages which disable it", "globalSetting", "globalSettingVal");     
  content += _buildRDFLiteralAsHTML(passwordMaker.globalSettingsSubjectRes, 
    passwordMaker.autoClearClipboardPredicateRes, "Auto-clear clipboard", "globalSetting", "globalSettingVal");     
  content += _buildRDFLiteralAsHTML(passwordMaker.globalSettingsSubjectRes, 
    passwordMaker.autoClearClipboardSecondsPredicateRes, "Number of seconds after which clipboard is cleared", "globalSetting", "globalSettingVal");
  content += _buildRDFLiteralAsHTML(passwordMaker.globalSettingsSubjectRes, 
    passwordMaker.statusbarIndicatorPredicateRes, "Show statusbar indicator", "globalSetting", "globalSettingVal");

  var literal = passwordMaker._getLiteral(passwordMaker.globalSettingsSubjectRes, passwordMaker.passwordShortcutPredicateRes);
  switch (literal) {
    case "pop-all" : literal = "Populate All Fields"; break;
    case "off" : literal = "Do nothing"; break;
    case "empty-only" : literal = "Populate empty fields only"; break;
    case "clear-all" : literal = "Clear all fields"; break;
  }
  return content + "<span class=\"globalSetting\">Action to take when CoolKey is activated</span><span class=\"globalSettingVal\">" + literal + "</span>";
}

function _printContainer(containerName, containerStr, includePwd, buildFcn) {
  var content = "<div class=\"title\">" + containerName + "</div>";
  try {
    if (PwdMkr_RDFUtils.isEmpty(containerStr))
      content += "<span class=\"emptyVal\">none</span><br/>";
    else {    
      var folders = PwdMkr_RDFUtils.makeSequence(containerStr).GetElements();
      while (folders.hasMoreElements()) {
        var folder = folders.getNext();
        if (folder instanceof Components.interfaces.nsIRDFResource) {
          content += _buildRDFLiteralAsHTML(folder, passwordMaker.namePredicateRes, "", "groupName", "groupNameVal");
          content += _buildRDFLiteralAsHTML(folder, passwordMaker.descriptionPredicateRes, "", "groupNotes", "groupNotesVal") + "<br/>";
          if (PwdMkr_RDFUtils.rdfContainerUtils.IsContainer(PwdMkr_RDFUtils.datasource, folder)) {
            var container = Components.classes["@mozilla.org/rdf/container;1"].createInstance(Components.interfaces.nsIRDFContainer);
            container.Init(PwdMkr_RDFUtils.datasource, folder);      
            var accounts = container.GetElements();
            while (accounts.hasMoreElements()) {
              var account = accounts.getNext();
              content += buildFcn(account, includePwd) + "<br/><br/>";
            }
          }
          else {
            // The defaults or any other top-level non-container resource
            content += buildFcn(folder) + "<br/><br/>"; // not really a folder!
          }        
        }
      }
    }
  }
  catch(e) {
    dump(e + "\n");
    content += "<span class=\"emptyVal\">none</span><br/>";
  }
  return content;
}

function _buildRemoteAsHTML(account, includePwd) {
  var content = "";
  account = account.QueryInterface(Components.interfaces.nsIRDFResource);
  content += _buildRDFLiteralAsHTML(account, passwordMaker.namePredicateRes, "Name", "remoteSetting", "remoteSettingVal");
  content += _buildRDFLiteralAsHTML(account, passwordMaker.descriptionPredicateRes, "Notes", "remoteSetting", "remoteSettingVal");
  content += _buildRDFLiteralAsHTML(account, PwdMkr_RDFUtils.makePredicate("urlToUse"), "URL", "remoteSetting", "remoteSettingVal", true);
  content += _buildRDFLiteralAsHTML(account, PwdMkr_RDFUtils.makePredicate("useWebDAV"), "Use WebDAV", "remoteSetting", "remoteSettingVal");  
  content += _buildRDFLiteralAsHTML(account, PwdMkr_RDFUtils.makePredicate("usernameTB"), "Username", "remoteSetting", "remoteSettingVal");  
  if (includePwd) {
    var encPassword = PwdMkr_RDFUtils.getTarget(account, "masterPassword", null); 
    var key = PwdMkr_RDFUtils.getTarget(account, "masterPasswordKey", null);     
    if (encPassword && key) {
      content += _buildAsHTML(PwdMkr_MPW.decrypt(encPassword, key),
        "Password", "remoteSetting", "remoteSettingVal");
    }
  }
  content += _buildRDFLiteralAsHTML(account, PwdMkr_RDFUtils.makePredicate("directory"), "Backup Directory", "remoteSetting", "remoteSettingVal");
  return content + _buildRDFLiteralAsHTML(account, PwdMkr_RDFUtils.makePredicate("file"), "File Pattern", "remoteSetting", "remoteSettingVal");
}

function _buildAccountAsHTML(account, includeGenPassword) {
  var content = "";
  account = account.QueryInterface(Components.interfaces.nsIRDFResource);
  if (account != passwordMaker.defaultSubjectRes) {
    content += _buildRDFLiteralAsHTML(account, passwordMaker.namePredicateRes, "Name", "accountSetting", "accountSettingVal");
    content += _buildRDFLiteralAsHTML(account, passwordMaker.descriptionPredicateRes, "Notes", "accountSetting", "accountSettingVal");
  }
  content += _buildRDFLiteralAsHTML(account, passwordMaker.whereLeetLBPredicateRes, "Use l33t", "accountSetting", "accountSettingVal");          
  content += _buildRDFLiteralAsHTML(account, passwordMaker.leetLevelLBPredicateRes, "l33t level", "accountSetting", "accountSettingVal");
  content += _buildRDFLiteralAsHTML(account, passwordMaker.hashAlgorithmLBPredicateRes, "Hash Algorithm", "accountSetting", "accountSettingVal");
  content += _buildRDFLiteralAsHTML(account, passwordMaker.passwordLengthPredicateRes, "Password Length", "accountSetting", "accountSettingVal");
  if (account == passwordMaker.defaultSubjectRes) {
    content += _buildRDFLiteralAsHTML(account, passwordMaker.protocolCBPredicateRes, "Protocol", "accountSetting", "accountSettingVal");          
    content += _buildRDFLiteralAsHTML(account, passwordMaker.subdomainCBPredicateRes, "Subdomain(s)", "accountSetting", "accountSettingVal");
    content += _buildRDFLiteralAsHTML(account, passwordMaker.domainCBPredicateRes, "Domain", "accountSetting", "accountSettingVal");
    content += _buildRDFLiteralAsHTML(account, passwordMaker.pathCBPredicateRes, "Port, path, anchor, query parameters", "accountSetting", "accountSettingVal");          
  }
  else {
    content += _buildRDFLiteralAsHTML(account, passwordMaker.urlToMatchPredicateRes, "URL To Match", "accountSetting", "accountSettingVal", true);          
    content += _buildRDFLiteralAsHTML(account, passwordMaker.urlToUsePredicateRes, "URL To Use", "accountSetting", "accountSettingVal", true);          
  }
  content += _buildRDFLiteralAsHTML(account, passwordMaker.usernameTBPredicateRes, "Username", "accountSetting", "accountSettingVal");                    
  content += _buildRDFLiteralAsHTML(account, passwordMaker.counterPredicateRes, "Modifier", "accountSetting", "accountSettingVal");
  content += _buildRDFLiteralAsHTML(account, passwordMaker.charsetPredicateRes, "Characters", "accountSetting", "accountSettingVal");
  content += _buildRDFLiteralAsHTML(account, passwordMaker.prefixPredicateRes, "Prefix", "accountSetting", "accountSettingVal");
  content += _buildRDFLiteralAsHTML(account, passwordMaker.suffixPredicateRes, "Suffix", "accountSetting", "accountSettingVal");
  content += _buildRDFLiteralAsHTML(account, passwordMaker.autoPopulatePredicateRes, "Automatic Population", "accountSetting", "accountSettingVal");                              
  if (includeGenPassword && account != passwordMaker.defaultSubjectRes) {    
    var genPW = passwordMaker._generateSiteSpecificPassword(account);
    content += "<span class=\"accountSetting\">Generated Password</span><span class=\"genPasswordVal\">";
    // Don't use CDATA sections if writing to HTML file
    content += isExporting ? genPW.password : ("<![CDATA[" + genPW.password + "]]>");
    content += "</span>";
  }
  return content;
}

function _buildRDFLiteralAsHTML(subject, predicate, predicateAsString, predicateClass, literalClass, isLink) {
  var literal;
  if (subject == null || predicate == null)
    literal = "none";
  else {
    var literal = passwordMaker._getLiteral(subject, predicate);
    if (literal == "false")
      literal = "unchecked";
    if (literal == "true")
      literal = "checked";
  }
  return _buildAsHTML(literal, predicateAsString, predicateClass, literalClass, isLink);
}

function _buildAsHTML(literal, predicateAsString, predicateClass, literalClass, isLink) {
  var ret =  "<span class=\"" + predicateClass + "\">" + predicateAsString + "</span>";
  if (!literal)
    ret += "<span class=\"emptyVal\">empty";
  else {  
    if (isLink) {
      if (isExporting) {
        // Make links clickable in HTML
        ret += literal.substring(0, 4) == "http" ?
          "<span class=\"" + literalClass + "\"><a href=\"" + literal + "\">" + literal + "</a>" : // already clickable
          "<span class=\"" + literalClass + "\"><a href=\"http://" + literal + "\">" + literal + "</a>"; // force clickable
      }
      else {
        // Links shouldn't be clickable so they use the right style
        ret += "<span class=\"" + literalClass + "\"><a href=\"" + literal + "\">" + literal + "</a>";
      }
    }
    else {
      if (isExporting) {
        // Don't use CDATA sections for HTML
        ret += "<span class=\"" + literalClass + "\">" + literal;
      }
      else {
        ret += "<span class=\"" + literalClass + "\"><![CDATA[" + literal + "]]>";
      }
    }
  }
  return ret + "</span><br/>";
}