/**
  PasswordMaker - Creates and manages passwords
  Copyright (C) 2005 Eric H. Jung and LeahScape, Inc.
  http://passwordmaker.org/
  grimholtz@yahoo.com

  This library is free software; you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by
  the Free Software Foundation; either version 2.1 of the License, or (at
  your option) any later version.

  This library is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESSFOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
  for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with this library; if not, write to the Free Software Foundation,
  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
**/

/** Common tree fcns **/

var accountsTree, remotesTree, domainsTree,
  copySelectionCmd, moveSelectionCmd;

function initTrees() {
  domainsTree = document.getElementById("domainsTree");  
  accountsTree = document.getElementById("accountsTree");
  remotesTree = document.getElementById("remotesTree");
  copySelectionCmd = document.getElementById("copySelectionCmd");
  moveSelectionCmd = document.getElementById("moveSelectionCmd");      
  _initTree(accountsTree);
  _initTree(remotesTree);
}

function _initTree(tree) {
  // Remove all datasources then add the single datasource.
  // This prevents the datasource being added multiple times
  // for example when user downloads passwordsmaker.rdf over and over
  var e = tree.database.GetDataSources();
  while (e.hasMoreElements())
    tree.database.RemoveDataSource(e.getNext());
  
  tree.database.AddDataSource(PwdMkr_RDFUtils.datasource);
  tree.builder.rebuild();
  if (tree.view)
    tree.view.selection.select(0);

  // Only add click listener ONCE--this method can be
  // called multiple times if user keeps downloading passwordmaker.rdf.
  tree.removeEventListener("click", clickListener, true);
  tree.addEventListener("click", clickListener, true);
}

function clickListener(event) {
  var row = {}, col = {}, childElt = {};
  this.treeBoxObject.getCellAt(event.clientX, event.clientY, row, col, childElt);
  if (event.detail == 2 && childElt.value != "twisty" && row.value > -1) {
    onAccountSettings(false);
    event.stopPropagation();
  }
}

function onTreeSelection() {
  var temp = getActiveTreeInfo();
  if (temp.tree.currentIndex == -1) {
    // Nothing selected. select the first item if there IS a first item.
    if (temp.tree.view.rowCount > 0)
      temp.tree.view.selection.select(0);
    return false;
  }  
  
  // if a folder...
  if (temp.tree.view.getItemAtIndex(temp.tree.currentIndex).getAttribute("container") == "true") {
    newAccountCmd.setAttribute("disabled", false); // Allow creation of new accounts 
    copySelectionCmd.setAttribute("disabled", true);   
    moveSelectionCmd.setAttribute("disabled", true);    
  }
  else {
    newAccountCmd.setAttribute("disabled", true); // Disallow creation of new accounts
    copySelectionCmd.setAttribute("disabled", false);
    moveSelectionCmd.setAttribute("disabled", false);    
  }
  return true;
}

function promptForTwoItems(name, desc) {
  var params = {inn: {name:name, description:desc}, out:null};

  window.openDialog("chrome://passwdmaker/chrome/treeprompt.xul",
    "", "chrome, dialog, modal, resizable=yes", params).focus();

  if (params.out) {
    // User clicked OK
    while (params.out && params.out.name == "") {
      // MUST enter a name
      passwordMaker.prompts.alert(this, "PasswordMaker", "A name is required.");
      params.out = null;
      window.openDialog("chrome://passwdmaker/chrome/treeprompt.xul", "", "chrome, dialog, modal, resizable=yes",
        params).focus();
    }
    return params.out;
  }
}

function createNewFolder(nameURI, descURI, containerSeq, folderRes, nameVal, descVal) {
  var folderInfo = promptForTwoItems(nameVal, descVal);
  if (!folderInfo)
    return; // user cancelled

  // Build the folder resource...
  // Folder Name
  if (!folderRes)
    folderRes = PwdMkr_RDFUtils.makeResource();
    
  PwdMkr_RDFUtils.updateTarget(folderRes, nameURI, folderInfo.name);
  PwdMkr_RDFUtils.updateTarget(folderRes, descURI, folderInfo.description);

  // Make the new sequence/container
  var newContainer = PwdMkr_RDFUtils.makeSequence(folderRes);
  // Append it to the parent container (e.g., the accounts sequence)
  containerSeq.AppendElement(folderRes);
  PwdMkr_RDFUtils.flush();
}

function onCreateNewFolder() {   
  createNewFolder("name", "description", getActiveTreeInfo().container);
}

function getActiveTreeInfo(tree) {
  var ret = new Object();
  ret.tree = tree;
  if (!ret.tree) { 
    switch (document.getElementById("tabs").selectedItem.label) {
      case "Accounts" : {
        ret.tree = accountsTree;
        ret.container = passwordMaker.accountsContainer
        ret.isAccounts = true;
        break;
      }
      case "Upload/Download" : {
        ret.tree = remotesTree;
        ret.container = passwordMaker.remotesContainer;
        ret.isRemotes = true;
        break;
      }
      case "Special Domains" : {
        ret.tree = domainsTree;
        ret.container = passwordMaker.domainsContainer
        ret.isDomains = true;
        break;
      }
    }
  }
  // Get the RDF resource associated with the selected tree item.
  // Check ret.tree.view before getting the currentIndex because,
  // if the tree is empty, ret.tree.currentIndex causes a "this.view has no properties"
  // JS error in chrome://global/content/bindings/tree.xml 
  if (ret.tree && ret.tree.view && ret.tree.currentIndex > -1) {
    // Get id of selected item in tree
    var subjectId = ret.tree.builderView.getItemAtIndex(ret.tree.currentIndex).getAttribute("id"); // e.g., rdf:#$JZPes
    ret.subjectRes = PwdMkr_RDFUtils.makeResource(subjectId);
    //dump("id = " + subjectId + "\n");
    ret.isContainer = PwdMkr_RDFUtils.rdfContainerUtils.IsContainer(PwdMkr_RDFUtils.datasource, ret.subjectRes);
  }
  return ret;
}

function onMoveAccount() {
  var temp = getActiveTreeInfo();

  var params = {inn:{passwordMaker:passwordMaker, copy:false, isAccounts:temp.tree==accountsTree}, out:null};

  window.openDialog("chrome://passwdmaker/chrome/folders.xul", "",
    "chrome, dialog, modal, resizable=yes", params).focus();

  if (params.out) {
    // User clicked OK
    var selectedContainerRes = PwdMkr_RDFUtils.rdfContainerUtils.MakeSeq(PwdMkr_RDFUtils.datasource, params.out.subjectRes);

    // Remove reference to this subject in whichever container contains it.
    var arcsIn = PwdMkr_RDFUtils.datasource.ArcLabelsIn(temp.subjectRes);
    while (arcsIn.hasMoreElements()) {
      var arc = arcsIn.getNext();
      if (arc instanceof Components.interfaces.nsIRDFResource) {
        if (PwdMkr_RDFUtils.rdfContainerUtils.IsOrdinalProperty(arc)) {
          var parentRes = PwdMkr_RDFUtils.datasource.GetSource(arc, temp.subjectRes, true);
          var parentResContainer = PwdMkr_RDFUtils.rdfContainerUtils.MakeSeq(PwdMkr_RDFUtils.datasource, parentRes);
          parentResContainer.RemoveElement(temp.subjectRes, true);
          break;
        }
      }
    }
    selectedContainerRes.AppendElement(temp.subjectRes);
    PwdMkr_RDFUtils.flush();
    if (temp.isAccounts)
      passwordMaker.loadURLs(); // refresh arrays    

    // Open the parent folder of the moved account if it's not already open.
    //if (!temp.tree.view.isContainerOpen(temp.tree.currentIndex))
      //temp.tree.view.toggleOpenState(temp.tree.currentIndex);
  }
}

function onCopyAccount() {
  var temp = getActiveTreeInfo();

  var params = {inn:{passwordMaker:passwordMaker, copy:true, isAccounts:temp.tree==accountsTree}, out:null};

  window.openDialog("chrome://passwdmaker/chrome/folders.xul", "",
    "chrome, dialog, modal, resizable=yes", params).focus();

  if (params.out) {
    // User clicked OK
    var newAccountRes = PwdMkr_RDFUtils.makeResource();
    if (temp.isAccounts) {
      PwdMkr_RDFUtils.updateTarget(newAccountRes, "name", passwordMaker._getLiteral(temp.subjectRes, passwordMaker.namePredicateRes));
      PwdMkr_RDFUtils.updateTarget(newAccountRes, "description", passwordMaker._getLiteral(temp.subjectRes, passwordMaker.descriptionPredicateRes));
      PwdMkr_RDFUtils.updateTarget(newAccountRes, "whereLeetLB", passwordMaker._getLiteral(temp.subjectRes, passwordMaker.whereLeetLBPredicateRes));
      PwdMkr_RDFUtils.updateTarget(newAccountRes, "leetLevelLB", passwordMaker._getLiteral(temp.subjectRes, passwordMaker.leetLevelLBPredicateRes));
      PwdMkr_RDFUtils.updateTarget(newAccountRes, "hashAlgorithmLB", passwordMaker._getLiteral(temp.subjectRes, passwordMaker.hashAlgorithmLBPredicateRes));
      PwdMkr_RDFUtils.updateTarget(newAccountRes, "passwordLength", passwordMaker._getLiteral(temp.subjectRes, passwordMaker.passwordLengthPredicateRes));
      PwdMkr_RDFUtils.updateTarget(newAccountRes, "urlToMatch", passwordMaker._getLiteral(temp.subjectRes, passwordMaker.urlToMatchPredicateRes));
      PwdMkr_RDFUtils.updateTarget(newAccountRes, "urlToUse", passwordMaker._getLiteral(temp.subjectRes, passwordMaker.urlToUsePredicateRes)); 
      PwdMkr_RDFUtils.updateTarget(newAccountRes, "usernameTB", passwordMaker._getLiteral(temp.subjectRes, passwordMaker.usernameTBPredicateRes));
      PwdMkr_RDFUtils.updateTarget(newAccountRes, "counter", passwordMaker._getLiteral(temp.subjectRes, passwordMaker.counterPredicateRes));
      PwdMkr_RDFUtils.updateTarget(newAccountRes, "charset", passwordMaker._getLiteral(temp.subjectRes, passwordMaker.charsetPredicateRes));
      PwdMkr_RDFUtils.updateTarget(newAccountRes, "prefix", passwordMaker._getLiteral(temp.subjectRes, passwordMaker.prefixPredicateRes));
      PwdMkr_RDFUtils.updateTarget(newAccountRes, "suffix", passwordMaker._getLiteral(temp.subjectRes, passwordMaker.suffixPredicateRes));
      PwdMkr_RDFUtils.updateTarget(newAccountRes, "autoPopulate", passwordMaker._getLiteral(temp.subjectRes, passwordMaker.autoPopulatePredicateRes));
      passwordMaker.loadURLs(); // refresh arrays       
    }
    else if (temp.isRemotes) {
      PwdMkr_RDFUtils.updateTarget(newAccountRes, "name", PwdMkr_RDFUtils.getTarget(temp.subjectRes, "name", ""));      
      PwdMkr_RDFUtils.updateTarget(newAccountRes, "description", PwdMkr_RDFUtils.getTarget(temp.subjectRes, "description", ""));        
      PwdMkr_RDFUtils.updateTarget(newAccountRes, "urlToUse", PwdMkr_RDFUtils.getTarget(temp.subjectRes, "urlToUse", ""));
      PwdMkr_RDFUtils.updateTarget(newAccountRes, "usernameTB", PwdMkr_RDFUtils.getTarget(temp.subjectRes, "usernameTB", ""));    
      PwdMkr_RDFUtils.updateTarget(newAccountRes, "masterPassword", PwdMkr_RDFUtils.getTarget(temp.subjectRes, "masterPassword", null));            
      PwdMkr_RDFUtils.updateTarget(newAccountRes, "masterPasswordKey", PwdMkr_RDFUtils.getTarget(temp.subjectRes, "masterPasswordKey", null));        
      PwdMkr_RDFUtils.updateTarget(newAccountRes, "useWebDAV", PwdMkr_RDFUtils.getTarget(temp.subjectRes, "useWebDAV", "false"));                    
      PwdMkr_RDFUtils.updateTarget(newAccountRes, "autoDownload", PwdMkr_RDFUtils.getTarget(temp.subjectRes, "autoDownload", "false"));        
      PwdMkr_RDFUtils.updateTarget(newAccountRes, "autoUpload", PwdMkr_RDFUtils.getTarget(temp.subjectRes, "autoUpload", "false"));                
      PwdMkr_RDFUtils.updateTarget(newAccountRes, "autoUploadWhenDirty", PwdMkr_RDFUtils.getTarget(temp.subjectRes, "autoUploadWhenDirty", "false"));                
      PwdMkr_RDFUtils.updateTarget(newAccountRes, "merge", PwdMkr_RDFUtils.getTarget(temp.subjectRes, "merge", "false")); 
      PwdMkr_RDFUtils.updateTarget(newAccountRes, "directory", PwdMkr_RDFUtils.getTarget(temp.subjectRes, "directory", PWDMKRDirIO.get("TmpD").path));                
      PwdMkr_RDFUtils.updateTarget(newAccountRes, PwdMkr_RDFUtils.getTarget(temp.subjectRes, "filename", ""));     
    }
              
    var containerRes = PwdMkr_RDFUtils.rdfContainerUtils.MakeSeq(PwdMkr_RDFUtils.datasource, params.out.subjectRes);
    // Add the resource to the container (folder)
    containerRes.AppendElement(newAccountRes);
    PwdMkr_RDFUtils.flush();

    // Open the selected item (the parent folder of the new account we just created)
    // if it's not already open.
    //if (!temp.tree.view.isContainerOpen(temp.tree.currentIndex))
      //temp.tree.view.toggleOpenState(temp.tree.currentIndex);     
  } 
}

/**
 * Called to create a new account in the currently selected folder.
 */
function onCreateNewAccount() {
  var temp = getActiveTreeInfo();
  var containerRes = PwdMkr_RDFUtils.makeSequence(temp.subjectRes);
  onAccountSettings(true);

  // Open the selected item (the parent folder of the new accounts we just created)
  // if it's not already open.
  if (!temp.tree.view.isContainerOpen(temp.tree.currentIndex))
    temp.tree.view.toggleOpenState(temp.tree.currentIndex);

  // Select the last item in the now-open folder (the newly-created account)
  //var newAccountId = document.getElementById(accountRes.Value);
  //var idx = temp.tree.view.getIndexOfItem(newAccountId);
  //if (idx >= 0)
    //temp.tree.view.selection.select(idx);
}

function deleteNodes(subject) {
  var isContainer = PwdMkr_RDFUtils.rdfContainerUtils.IsContainer(PwdMkr_RDFUtils.datasource, subject);
  if (isContainer) {
    // Subject is a container (folder)
    var container = PwdMkr_RDFUtils.rdfContainerUtils.MakeSeq(PwdMkr_RDFUtils.datasource, subject);
    var children = container.GetElements();
    while (children.hasMoreElements()){
      var child = children.getNext();
      if (child instanceof Components.interfaces.nsIRDFResource){
        container.RemoveElement(child, true);
        children = container.GetElements();
        prune(child);
        // Delete fields container and elements, if they exist
        child = PwdMkr_RDFUtils.makeResource(child.Value + "/fields");
        deleteNodes(child);
      }
    }
  }
  else {
    // Not a container (folder). Remove reference
    // to this subject in whichever container contains it.
    var arcsIn = PwdMkr_RDFUtils.datasource.ArcLabelsIn(subject);
    while (arcsIn.hasMoreElements()) {
      var arc = arcsIn.getNext();
      if (arc instanceof Components.interfaces.nsIRDFResource) {
        if (PwdMkr_RDFUtils.rdfContainerUtils.IsOrdinalProperty(arc)) {
          var parentRes = PwdMkr_RDFUtils.datasource.GetSource(arc, subject, true);
          var parentResContainer = PwdMkr_RDFUtils.rdfContainerUtils.MakeSeq(PwdMkr_RDFUtils.datasource, parentRes);
          parentResContainer.RemoveElement(subject, true);
        }
      }
    }
  }
  prune(subject);
  PwdMkr_RDFUtils.flush();
}

function prune(subject) {
  // Remove all remaining tuples
  var targets = PwdMkr_RDFUtils.datasource.ArcLabelsOut(subject);
  while (targets.hasMoreElements()){
    var predicate = targets.getNext();
    if (predicate instanceof Components.interfaces.nsIRDFResource)
      PwdMkr_RDFUtils.removeTarget(subject, predicate);
  }
}

function deleteTreeItem() {
  var temp = getActiveTreeInfo();

  if (!passwordMaker.ask(this, temp.isContainer?"Are you sure you want to delete this item and all items below it?" :
      "Are you sure you want to delete this item?"))
    return false;

  deleteNodes(temp.subjectRes);  
  temp.container.RemoveElement(temp.subjectRes, true); // remove from the container
  PwdMkr_RDFUtils.flush();
  return true;
}

function onDeleteTreeItem() {
  var temp = getActiveTreeInfo();
  
  if (temp.isAccounts && temp.tree.currentIndex == 0)
    passwordMaker.prompts.alert(this, "PasswordMaker", "The default settings cannot be deleted.");
  else if (deleteTreeItem() && temp.isAccounts) {
    // This chunk is only necessary for the accountsTree
    // since only *it* can contain a fields container
    // Delete fields container and elements, if they exist
    temp.subjectRes.QueryInterface(Components.interfaces.nsIRDFResource);
    temp.subjectRes = PwdMkr_RDFUtils.makeResource(temp.subjectRes.Value + "/fields");
    deleteNodes(temp.subjectRes);
    PwdMkr_RDFUtils.flush();
    passwordMaker.loadURLs(); // refresh arrays   
  }
}

function getTreeCells(tree, index) {
  if (index == null)
    index = tree.currentIndex;
  var selectedTreeItem = tree.view.getItemAtIndex(index);   // <treeitem/>
  var selectedTreeRow = selectedTreeItem.firstChild;   // <treerow/>
  return selectedTreeRow.childNodes;  // <treecell/> NodeList
}